#include "grafo.h"
#include <stdlib.h>

#ifdef DEBUG
    #include <stdio.h>
#endif

nodo *crear_nodo(void *p, LISTA *ady) {

     nodo *n = (nodo *)malloc(sizeof(nodo));
     n->ady = ady;
     n->p = p;
     return n;

}

/*
 * Este algoritmo puede ser un poco pesado de digerir, asi que
 * hay que ponerle atención. Preguntemonos primero qué cosas no
 * deberíamos hacer al liberar la memoria del grafo. Estas son:
 *      
 *  - No liberar dos veces un mismo nodo. Esto podría ocurrir
 *    debido a que en el grafo se puede llegar de más de 2 formas
 *    al nodo.
 *  - Pueden quedar nodos sin liberar. Esto ocurre porque al
 *    liberar un nodo se hace imposible acceder a traves de él a sus
 *    vecinos. Por lo tanto si a traves de él era la unica forma de
 *    acceder a un nodo que no ha sido liberado, este último quedará
 *    sin liberar.
 * 
 * La forma de solucionar lo anterior es ir guardando en un conjunto
 * (S) los nodos ya visitados, y por cada nodo llamar recursivamente
 * a la función para liberar memoria sólo a los hijos que no estan en
 * S. Una forma de ver que esto funciona es notar que esta forma de
 * recorrer el grafo genera un arbol cobertor mínimo del grafo, y es
 * fácil liberar la memoria en un árbol pues no tenemos los problemas
 * anteriores.
 * En pseudo código el algoritmo sería:
 * free(n, S)
 *     while Existe v in ADY\S
 *         free(v, S u {v})
 *     free(n);
 * Ahora toda la dificultad esta en el "Existe", pero no es tan peluo
 * son los 2 for...        
 */
static LISTA *eliminar_nodo(nodo *n, LISTA *visitados) {

    nodo *u, *v;
    int is_in;
    for(u = (nodo *)first_l(n->ady); u != NULL; u = (nodo *)next_l(n->ady)) {
        is_in = 0;
        for(v = (nodo *)first_l(visitados); v != NULL; v = (nodo *)next_l(visitados)) {
            if(u == v){
                is_in = 1;
                break;
            }
        }
        if(!is_in) 
            visitados = eliminar_nodo(u, cons_l(u, visitados));
    }
    free_l(n->ady);
    #ifdef DEBUG
        printf("Liberando %s\n", (char *)n->p);
    #endif
    free(n);
    return visitados;

}

/*
 * Esto ta medio enredado, pero tomando en cuenta que los argumentos
 * de las funciones se evalúan antes que la función es facil cachar
 * que ṕasa
 */
void liberar_nodo(nodo *n) {

     free_l(eliminar_nodo(n, cons_l(n, NULL)));

}
