#include <stdio.h>
#include <stdlib.h>
#include "lista.h"

/*
 * Si la lista == NULL pedimos memoria con malloc. La memoria
 * que nos da malloc puede venir sucia, ie l->h y l->t pueden
 * apuntar a cualquier tontera. Por lo que es necesario hacer
 * l->h = NULL y l->NULL.
 * Luego, independiente de como era l, creamos el nodo de modo
 * tal que su siguiente sea el priemro de la lista. Finalmente
 * el nuevo primero de la lista es n y reseteamos el puntero
 * "móvil" al primero tb (esto es arbitrario).
 */
LISTA *cons_l(void *p, LISTA *l){
	
	struct nodo* n;
	if(l == NULL){
		l = (LISTA *)malloc(sizeof (LISTA));
		if(l == NULL)
			return l;
		l->h = NULL;
		l->t = NULL;
	}
	n = (struct nodo *)malloc(sizeof (struct nodo));
	n->next = l->h;
	n->val = p;
	l->h = n;
	l->t = n;
	return l;

}

void *first_l(LISTA *l){

	if(l == NULL)
		return NULL;
	l->t = l->h;
	return l->h->val;

}

/*
 * Ahora voy a explicar pa que sirve el l->h o puntero móvil.
 * La idea es ocupar las funciones first_l y next_l de la
 * siguiente forma:
 *     for(n = first_l(l); n != NULL; n = next_l(l))
 * y con ello recorremos la lista desde el primer hasta al
 * último elemento. Para hacer eso posicionamos l->t en la
 * cabeza con first_l y con next_l avanza una posición.
 */
void *next_l(LISTA *l){

	if(l == NULL)
		return NULL;
	if(l->t == NULL)
		return NULL;
	l->t = l->t->next;	
	return l->t == NULL? NULL:l->t->val;

}

/*
 * Es importante notar el orden en que se libera la lista, pues
 * sería erroneo eliminar primero la cabeza. El mismo princio, de
 * una forma más compleja, es el que se aplica en la p2.
 */
void free_l(LISTA *l){
	
	if(l != NULL){
		free_nodo(l->h);
		free(l);
	}

}

/*para poder escribir recursivamente la liberación de nodos*/
void free_nodo(struct nodo *n){
	
	if(n == NULL)
		return;
	free_nodo(n->next);
	free(n);

}
